/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "PConfig.h"
#include "DFT.hh"
#include "TSeries.hh"
#include "DVecType.hh"
#include "constant.hh"
#include "fft.hh"
#include <math.h>

using namespace containers;

//======================================  DFT default constructor
DFT::DFT(void) {
}

//======================================  DFT data constructor
DFT::DFT(const TSeries& ts) {
}

//======================================  DFT data constructor
DFT::DFT(const fSeries& fs) 
    : fSeries(fs)
{}

//======================================  DFT destructor
DFT::~DFT(void) {
}

//======================================  DFT clone method
DFT* 
DFT::clone(void) const {
    return new DFT(*this);
}

//======================================  DFT clone method
DFT
DFT::evolve(Interval dt) const {

    //----------------------------------  Get fSeries Parameters
    size_type N  = size();
    DVecType<dComplex>* dv = new DVecType<dComplex>(N);
    double t2pi = twopi * dt;

    //----------------------------------  Build up multiplier vector
#ifdef USE_SINCOS
    double sinphi, cosphi;
#endif
    for (size_type i=0; i<N; ++i) {
	double arg = getBinF(i) * t2pi;
#ifdef USE_SINCOS
	sincos(arg, &sinphi, &cosphi);
#else
	double sinphi = sin(arg);
	double cosphi = cos(arg);
#endif
	(*dv)[i] = dComplex(sinphi, cosphi);
    }

    //----------------------------------  Build the DFT
    DFT r(fSeries(getBinF(0), getFStep(), getStartTime(), getDt(), dv));
    r *= *this;

    return r;
}

//======================================  Set the data to the FFT of a TSeries
void
DFT::setData(const TSeries& ts) {
    clear();
    size_type nData = ts.getNSample();
    if (!nData) return;

    freq_type fNy = 0.5/ts.getTStep();
    freq_type f0  = ts.getF0() - fNy;
    Interval  dT  = ts.getInterval();
    freq_type dF  = 1.0/dT;
    Time      t0  = ts.getStartTime();

    setTimeSpan(t0, dT);
    const DVector* tsVect = ts.refDVect();
    size_type nOut(nData+1);

    //----------------------------------  Set the vector size.
    DVecType<dComplex>* dv = 0;
    if (!null() && refDVect().W_data()) {
	dv = dynamic_cast< DVecType<dComplex>* >(&refDVect());
	reserve(nOut);
    } else {
	dv = new DVecType<dComplex>(nOut);
    }

    //----------------------------------  Fill from a Complex TSeries.
    if (tsVect->C_data() || tsVect->W_data()) {
	*dv = *(ts.refDVect());
	wfft(dv->refTData(), nData, 1);

    } else if (tsVect->D_data()) {
	wfft(reinterpret_cast<const double*>(ts.refData()), dv->refTData(), 
	     nData);

    } else {
        ts.getData(nData, reinterpret_cast<double*>(dv->refData()));
	wfft(reinterpret_cast<double*>(dv->refData()), dv->refTData(), 
	     nData);
    }

    //------------------------------  Reorder the output vector.
    size_type N2 = nData/2;
    register dComplex *pP = dv->refTData() + N2;
    register dComplex *pN = dv->refTData();
    for (size_type i=0; i<N2; i++) {
	register dComplex t = *pN;
	*pN++ = *pP;
	*pP++ = t;
    }

    //-------------------------------  Normalize the data
    *dv *= ts.getTStep()/ts.getSigmaW();
    fSeries::setData(f0, dF, dv);
}
