#include <ctype.h>
#include <string.h>
#include <strings.h>
#include "filterwiz/FilterModule.hh"
#include "IIRFilter.hh"
#include "iirutil.hh"
#include <string>
#include <iostream>


   using namespace std;
   using namespace ligogui;

namespace filterwiz {


//////////////////////////////////////////////////////////////////////////
//                                                                      //
// FilterModule							        //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
   FilterModule::FilterModule ()
   : fSample (1)
   {
      for (int i = 0; i < kMaxFilterSections; ++i) {
         fSect.push_back (FilterSection (fSample, i));
      }
   }

//______________________________________________________________________________
   FilterModule::FilterModule (const char* name, double fsample)
   : fName (name ? name : ""), fSample (fsample)
   {
      for (int i = 0; i < kMaxFilterSections; ++i) {
         fSect.push_back (FilterSection (fSample, i));
      }
   }

//______________________________________________________________________________
   void FilterModule::checkDesign()
   {
      int split = 0;
   
      for (int i = kMaxFilterSections - 1; i >= 0; --i) {
         if (fSect[i].getDesign() == "split") {
            ++split;
            continue;
         }
         // get filter from design string
         IIRFilter iir_design;
         FilterDesign design (fSample);
         bool ok = design.filter (fSect[i].getDesign());
         if (ok) {
            try {
               iir_design = iir2iir (design.get());
            } 
               catch (...) {
                  ok = false;
               }
         }
         // get filter from coefficients
         IIRFilter iir_coeff;
         if (ok) {
            for (int j = i; j <= i + split; ++j) {
               try {
                  IIRFilter iir = iir2iir (fSect[j].filter().get());
                  iir_coeff *= iir;
               } 
                  catch (...) {
                     ok = false;
                     break;
                  }
            }
         }
         // now compare the two
         if (ok) {
            ok = iircmp (iir_design, iir_coeff);
         }
         // if not ok, patch in some names
         if (!ok) {
            cerr << "Filer module " << getName() << " section " << i <<
               ": mismatch between design and coefficients" << endl;
            for (int j = i; j <= i + split; ++j) {
               string cmd;
               if (iir2zpk (fSect[j].filter().get(), cmd, "n") ||
                  iir2z (fSect[j].filter().get(), cmd, "s")) {
                  fSect[j].setDesign (cmd.c_str());
               }
               else {
                  fSect[j].setDesign ("");
                  fSect[j].filter().reset();
               }
            }
         }
         split = 0;
      }
   }

//______________________________________________________________________________
   void FilterModule::setName (const char* p) 
   {
      fName = p;
      string::size_type pos;
      while ((pos = fName.find_first_of (" \n\f\r\t\v")) != string::npos) {
         fName.erase (pos, 1);
      }
   }

//______________________________________________________________________________
   void FilterModule::setFSample (double sample) 
   {
      fSample = sample; 
      for (int i = 0; i < kMaxFilterSections; ++i) {
         fSect[i].filter().setFSample (fSample);
      }
   }

//______________________________________________________________________________
   void FilterModule::guessFSample (const char* filename) 
   {
      while (filename && strchr (filename, '/')) {
         filename = strchr (filename, '/') + 1;
      }
      // is filename LHO, LLO, CIT or MIT?
      if (!filename || (strchr ("HLMC", filename[0]) == 0) ||
         (strlen (filename) < 5) || !isdigit (filename[1])) {
         setFSample (1);
      }
      // check subsystem
      if (strncasecmp (filename + 2, "LSC", 3) == 0) {
         setFSample (16384);
      }
      else if (strncasecmp (filename + 2, "ASC", 3) == 0) {
         setFSample (2048);
      }
      else if (strncasecmp (filename + 2, "IOO", 3) == 0) {
         setFSample (2048);
      }
      else if (strncasecmp (filename + 2, "SEI", 3) == 0) {
         setFSample (2048);
      }
      else if (strncasecmp (filename + 2, "SUS", 3) == 0) {
         const char* p = filename + 5;
         if (*p && (p[0] == '_')) ++p;
         bool etm = (strncasecmp (p, "ETM", 3) == 0);
         bool fast = 
            (strncasecmp (p, "ETM", 3) == 0) ||
            (strncasecmp (p, "ITM", 3) == 0) ||
            (strncasecmp (p, "FM", 2) == 0) ||
            (strncasecmp (p, "BS", 2) == 0) ||
            (strncasecmp (p, "RM", 2) == 0) ||
            (strncasecmp (p, "MC2", 3) == 0);
         p = (fName.size() > 2) ? fName.c_str() + 2 : 0;
         bool sens = (*p && (strcasecmp (p, "SEN") == 0));
         if (!fast || (sens && !etm)) {
            setFSample (2048);
         }
         else {
            setFSample (16384);
         }
      }
      else {
         setFSample (2048);
      }
   }

}
