#ifndef LMSG_MSGHANDLER_HH
#define LMSG_MSGHANDLER_HH
//
//   Base class for server message handlers.
//

#include "lmsg/MsgTypes.hh"

namespace lmsg {
  
  class MsgHeader;
  class Message;
  class AppServer;
  class Buffer;

  class MsgHandler {
  public:
    MsgHandler(void);
    virtual ~MsgHandler(void);
    error_type handleBuffer(AppServer& app, Buffer& buf);
    virtual error_type handleMsg(AppServer& app, const MsgHeader& hdr) = 0;
    virtual Message& getMessage(void) = 0;
  private:
  };


  template<class Msg> 
  class Handler : public MsgHandler {
  public:
      Handler(void);
      ~Handler(void);
      error_type handleMsg(AppServer& app, const MsgHeader& hdr);
      Msg& getMessage(void);
      const Msg& getMessage(void) const;
  private:
      Msg mMessage;
  };

}  // namespace lmsg

//--------------------------------------  Default constructor
template<class Msg>
lmsg::Handler<Msg>::Handler(void) {
}

//--------------------------------------  Default destructor
template<class Msg>
lmsg::Handler<Msg>::~Handler(void) {
}

//--------------------------------------  Get the scratch message
template<class Msg>
Msg&
lmsg::Handler<Msg>::getMessage(void) {
    return mMessage;
}

template<class Msg>
const Msg&
lmsg::Handler<Msg>::getMessage(void) const {
    return mMessage;
}


#endif  //  LMSG_MSGHANDLER_HH


