#ifndef TRANSLATE_HH
#define TRANSLATE_HH

/**  Translate is a template class that maps the charater set to a set
  *  of enumerated values.
  *  @author   J. Zweizig
  *  @version  1.1; Last Modified August 5, 2004
  */
template <class T> 
class Translate {
public:
  /**  Construct an empty translate table.
    */
  Translate(void);

  /**  Construct a translate table and initialize all characters to the
    *  specified value.
    */
  Translate(T init);

  /**  Destry the translate table.
   */
  ~Translate(void);

  /** Set the table to translate the specified characters to listed value.
    */
  void char_type(const char*, T val);

  T operator[](char c) const;
  T& operator[](char c);
  void reset(void);
  void set_alpha(T alfval);
  void set_numer(T numval);
  void set_type(const char*, T val);
  void unset(T alfval);
private:
#ifndef __CINT__
  static const int table_count=256;
  static const int table_mask=table_count-1;
#else
  #define table_count 256
  #define table_mask  255
#endif
  T _initval;
  T _table[table_count];
};

template <class T> 
inline 
Translate<T>::Translate(void) 
  : _initval(T(0))
{
}

template <class T> 
inline 
Translate<T>::Translate(T init) 
  : _initval(init)
{
    reset();
}

template <class T> 
inline 
Translate<T>::~Translate(void) {
}

template <class T> 
inline T&
Translate<T>::operator[](char c) {
    return _table[int(c) & table_mask];
}

template <class T> 
inline T
Translate<T>::operator[](char c) const {
    return _table[int(c) & table_mask];
}

template <class T> 
inline void
Translate<T>::reset(void) {
    for (int i=0; i<table_count; ++i) _table[i] = _initval;
}

template <class T> 
inline void
Translate<T>::unset(T val) {
    for (int i=0; i<table_count; ++i) if (_table[i] == val) _table[i] = _initval;
}

template <class T> 
inline void
Translate<T>::char_type(const char* ccc, T val) {
    const char* p=ccc;
    while (*p) (*this)[*p++] = val;
}

template <class T> 
inline void
Translate<T>::set_type(const char* ccc, T val) {
    unset(val);
    char_type(ccc, val);
}

template <class T> 
inline void
Translate<T>::set_alpha(T val) {
    char_type("abcdefghijklmnopqrstuvwxyz_ABCDEFGHIJKLMNOPQRSTUVWXYZ", val);
}

template <class T> 
inline void
Translate<T>::set_numer(T val) {
    char_type("0123456789", val);
}

#endif // !defined(TRANSLATE_HH)
