#ifndef _OSMBUF_HH_
#define _OSMBUF_HH_
//
//   Shared memory output stream buffer class
//
#include "PConfig.h"
#if defined(__GNU_STDC_OLD)
#include <streambuf.h>
#else
#include <iostream>
#include <streambuf>
#endif

class LSMP_PROD;

/**  This class interfaces the shared memory buffer manager to the output
  *  stream class. It is based on the standard template class streambuf,
  *  and implements the virtual buffer management functions of streambuf.
  *  @memo Shared memory stream class.
  */
class oSMbuf : public std::streambuf {
  public:

    /**  Crate an unopened stream buffer.
      *  @memo Default creator.
      */
    oSMbuf();

    /**  Create and open a stream buffer.
      *  A stream buffer created and attached to a shared memory partition. 
      *  Attaching the stream buffer to the memory partition is described
      *  in the discussion of open().
      */
#if defined(__GNU_STDC_OLD)
    oSMbuf(const char *partition, std::ios::open_mode mode=std::ios::out);
#else
    oSMbuf(const char *partition, std::ios::openmode mode=std::ios::out);
#endif

    /**  Detach (if necessary) and destroy a stream buffer object.
      */
    ~oSMbuf();

    /**  Attach a stream buffer object to a shared memory partition.
      *  The shared memory partition "partition" is found, or created if
      *  it does not already exist. A data producer is created for the 
      *  partition to be used by the stream buffering functions. Buffering 
      *  of the stream uses the shared memory directly.
      */
#if defined(__GNU_STDC_OLD)
    oSMbuf* open(const char *partition, std::ios::open_mode mode=std::ios::out);
#else
    oSMbuf* open(const char *partition, std::ios::openmode mode=std::ios::out);
#endif

    /**  Implementation of virtual streambuf::doallocate.
      */
    int doallocate(void);

    /**  Implementation of virtual streambuf::overflow.
      */
    int overflow(int c);

    /**  Implementation of virtual streambuf::underflow.
      */
    int underflow() {return EOF;}
    int underflow(int c) {return EOF;}

    /**  Implementation of virtual streambuf::seekoff.
      */
#ifdef __GNU_STDC_OLD
    std::streampos seekoff(std::streamoff off, std::ios::seek_dir dir, 
			   std::ios::open_mode mode);
#else			   
    std::streampos seekoff(std::streamoff off, std::ios::seekdir dir, 
                           std::ios::openmode mode);
#endif
    /**  Implementation of virtual streambuf::sync.
      */
    int sync(void);

#ifndef __GNU_STDC_OLD
  void setb(char* b, char* eb, int a=0) {}
  int out_waiting(void) {return pptr() - pbase();}
#endif

  private:
    ///                   Point to the shared memory producer interface.
    LSMP_PROD *mProducer;
    ///                   Pointer to the buffer currently being written
    char      *mBuffer;
    ///                   Length of buffer currently being written.
    int        mLength;
};
#endif  //  _OSMBUF_HH_
